@@ -1,15 +1,13 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 """
-Created on Mon Feb 17 17:00:35 2020
-
-@author: jezequel
+Tests for cars data
 """
-
 import pkg_resources
-
 from dessia_common.vectored_objects import Catalog, ParetoSettings, Objective, ObjectiveSettings
-from dessia_common import DessiaFilter
+from dessia_common.core import DessiaFilter
+from dessia_common.tests import Car, CarWithFeatures
+from dessia_common.files import StringFile
 
 choice_args = ['MPG', 'Cylinders', 'Displacement', 'Horsepower', 'Weight', 'Acceleration', 'Model']  # Ordered
 
@@ -20,19 +18,22 @@ coefficients = {'Cylinders': 0, 'MPG': -0.70, 'Displacement': 0,
 pareto_settings = ParetoSettings(minimized_attributes=minimized_attributes, enabled=True)
 
 csv_cars = pkg_resources.resource_stream('dessia_common', 'models/data/cars.csv')
+stream = StringFile.from_stream(csv_cars)
 
+csv_cars.seek(0)
 catalog = Catalog.from_csv(csv_cars)
 catalog.name = 'Cars dataset'
 catalog.pareto_settings = pareto_settings
 
-filters = [
-    DessiaFilter(attribute="Weight", operator="lt", bound=4000)
-]
+filters = [DessiaFilter(attribute="Weight", comparison_operator="lt", bound=4000)]
 
 # Empty objective because this hasn't been used/tested for a while
 objective_settings = ObjectiveSettings()
 objective = Objective({}, {}, objective_settings)
 
-filtered_array = catalog.filter_(filters)
-filtered_catalog = Catalog(array=filtered_array, variables=catalog.variables, objectives=[objective])
+filtered_catalog = catalog.filter_(filters)
 merged_catalog = Catalog.concatenate(catalogs=[catalog, filtered_catalog])
+
+# Used models
+all_cars_no_feat = Car.from_csv(stream)
+all_cars_wi_feat = CarWithFeatures.from_csv(stream)
