@@ -1,3 +1,7 @@
+"""
+typings for dessia_common
+
+"""
 from typing import TypeVar, Generic, Dict, Any, Tuple
 
 
@@ -43,48 +47,3 @@ class ClassAttributeType(AttributeType[T]):
 # Types Aliases
 JsonSerializable = Dict[str, Any]
 RGBColor = Tuple[float, float, float]
-
-
-# Measures
-class Measure(float):
-    units = ''
-
-
-class Distance(Measure):
-    units = 'm'
-
-
-class Torque(Measure):
-    units = 'Nm'
-
-
-class Stress(Measure):
-    units = 'Pa'
-
-
-class Time(Measure):
-    units = 's'
-
-
-class Speed(Measure):
-    units = 'm/s'
-
-
-class Acceleration(Measure):
-    units = 'm/s²'
-
-
-class Mass(Measure):
-    units = 'Kg'
-
-
-class Force(Measure):
-    units = 'N'
-
-
-class Work(Measure):
-    units = 'N*m'
-
-
-class Power(Measure):
-    units = 'N*m/s'
