@@ -23,7 +23,7 @@ else:
 
 test_passed = license_should_be_valid
 try:
-    from dessia_common.core import prettyname
+    from dessia_common.utils.helpers import prettyname
     prettyname('aa')
     print('No license error triggered')
 except RuntimeError:
@@ -34,5 +34,5 @@ except RuntimeError:
     else:
         print('License was expected to be unvalid, test passed')
         test_passed = True
-    
-assert test_passed
\ No newline at end of file
+
+assert test_passed
